#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
class FloatMatrix{
public:	
	float **item; //contains the contents of the matrix
	int columns;
	int rows;
	
	/** Allocates memory and sets FloatMatrix values to 0.0*/
	FloatMatrix(int rows, int columns){
		this->columns=columns;
		this->rows=rows;
		
		item=new float*[rows];
		for(int i=0;i < rows; i++){
			item[i]=new float[columns];
			for (int j=0; j<columns; j++){
				item[i][j]=0.0;
			}
		}
	}
	
	/** DeAllocates memory*/
	~FloatMatrix(){
		
		for(int i=0;i < rows; i++){
			delete(item[i]);
			
		}
		delete(item);
	}
	
	/** Prints matrix out to stdout */
	void print(){
		for(int i=0;i < rows; i++){
			for (int j=0; j<columns; j++){
				cout<<item[i][j]<<" ";
			}
			cout<<endl;
		}
	}
	
	/** Copies matrix contents to current if rows & columns equal.
	    Returns TRUE if operation successful
	 */
	bool copyFrom(FloatMatrix *C){
		if (C->columns==columns && C->rows==rows){
			for (int i=0; i<rows;i++){
				for(int j=0; j<columns;j++){
					item[i][j]=C->item[i][j];
				}
			}
			return true;
		}else{
			cerr<<"Matrix size not equal in copy"<<endl;
			return false;
		}
	}
	
	void FloatMatrix::applyWeight(const float weight){
 	   for(int i=0; i<rows;i++){
		for(int j=0; j< columns;j++){
		    item[i][j]=item[i][j]*weight;
		}
	    }
	}
};
/** Returns a FloatMatrix with no. of rows and columns same as FloatMatrix B.
    A is the kernel matrix -- must have an odd number of rows and columns. Else result is undefined.
 
    edges are handled by wrapping around.
 */
FloatMatrix *convolve(FloatMatrix *A, FloatMatrix *B){
	FloatMatrix *Result=new FloatMatrix(B->rows, B->columns);
	short kernelCenterX = (A->columns - 1) / 2;
	short kernelCenterY = (A->rows - 1) / 2;
	int   imageRow, imageCol;
  	

	for(int i=0; i< B->rows;i++){
		for(int j=0; j< B->columns;j++){
			Result->item[i][j]=0.0;
			
			for (int a=0; a < A->rows; a++){
				for (int b=0; b< A->columns; b++){
					imageRow = (a - kernelCenterY) + i;
					imageRow = (imageRow < 0) ? B->rows + imageRow : imageRow % B->rows;
					imageCol = (b - kernelCenterX) + j;
					imageCol = (imageCol < 0) ? B->columns + imageCol : imageCol % B->columns;
					
					Result->item[i][j]+=B->item[imageRow][imageCol] * A->item[a][b];
				}
			}
		}
	}
	return Result;
}
